@@ -2,7 +2,7 @@
 # utils module - common functions for reportbug UIs
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2017 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2019 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -27,7 +27,7 @@ import platform
 
 try:
     import pwd
-    from .tempfiles import TempFile, tempfile_prefix, cleanup_temp_file
+    from .tempfiles import TempFile, cleanup_temp_file
 except ImportError as e:
     if platform.system() == 'Windows':
         pass
@@ -43,7 +43,6 @@ import apt
 import gzip
 
 from .urlutils import open_url
-from string import ascii_letters, digits
 
 # Paths for dpkg
 DPKGLIB = '/var/lib/dpkg'
@@ -227,7 +226,7 @@ def find_package_for(filename, pathonly=False):
 
     # tries to match also files in /var/lib/dpkg/info/
     if filename.startswith('/var/lib/dpkg/info/'):
-        dpkg_info = re.compile('/var/lib/dpkg/info/(.+)\.[^.]+')
+        dpkg_info = re.compile(r'/var/lib/dpkg/info/(.+)\.[^.]+')
         m = dpkg_info.match(filename)
         # callee want a dict as second pair element...
         packages[m.group(1)] = ''
@@ -248,7 +247,7 @@ def find_rewritten(username):
     for filename in ['/etc/email-addresses']:
         if os.path.exists(filename):
             try:
-                fp = open(filename)
+                fp = open(filename, errors='backslashreplace')
             except IOError:
                 continue
             for line in fp:
@@ -525,16 +524,6 @@ def get_avail_database():
     return AvailDB(popenob=subp)
 
 
-def available_package_description(package):
-    data = get_command_output('apt-cache show ' + pipes.quote(package))
-    descre = re.compile('^Description(?:-[a-zA-Z]+)?: (.*)$')
-    for line in data.split('\n'):
-        m = descre.match(line)
-        if m:
-            return m.group(1)
-    return None
-
-
 def get_source_name(package):
     packages = []
 
@@ -552,6 +541,8 @@ def get_source_package(package):
     retlist = []
     found = {}
 
+    apt_cache = apt.Cache()
+
     data = get_command_output('apt-cache showsrc ' + pipes.quote(package))
     binre = re.compile(r'^Binary: (.*)$')
     for line in data.split('\n'):
@@ -562,7 +553,10 @@ def get_source_package(package):
             packages += packlist
 
     for p in packages:
-        desc = available_package_description(p)
+        try:
+            desc = apt_cache[p].versions[0].summary
+        except KeyError:
+            continue
         if desc and (p not in found):
             retlist += [(p, desc)]
             found[p] = desc
@@ -613,9 +607,10 @@ def get_package_info(packages, skip_notfound=False):
                 sinfo = stat.split()
                 stat = sinfo[0][0] + sinfo[2][0]
                 # check if the package is installed, and in that case, retrieve
-                # its information; if the first char is not 'i' (installed) or
-                # the second is 'n' (not-installed), then skip data retrieval
-                if stat[0] != 'i' or stat[1] == 'n':
+                # its information; if the first char is not 'i' or 'h' (install
+                # or hold) or the second is 'n' (not-installed), then skip data
+                # retrieval
+                if stat[0] not in 'ih' or stat[1] == 'n':
                     continue
 
                 if m.group('hdr') == 'Provides':
@@ -716,7 +711,7 @@ def get_changed_config_files(conffiles, nocompress=False):
     changed = []
     for (filename, md5sum) in conffiles:
         try:
-            fp = open(filename)
+            fp = open(filename, errors='backslashreplace')
         except IOError as msg:
             confinfo[filename] = msg
             continue
@@ -751,7 +746,7 @@ def get_debian_release_info():
     dists = []
     output = get_command_output('apt-cache policy 2>/dev/null')
     if output:
-        mre = re.compile('\s+(\d+)\s+.*$\s+release\s.*o=(Ubuntu|Debian|Debian Ports),a=([^,]+),', re.MULTILINE)
+        mre = re.compile(r'\s+(\d+)\s+.*$\s+release\s.*o=(Ubuntu|Debian|Debian Ports),a=([^,]+),', re.MULTILINE)
         found = {}
         # XXX: When Python 2.4 rolls around, rewrite this
         for match in mre.finditer(output):
@@ -771,9 +766,8 @@ def get_debian_release_info():
             debvers = dists[0][1]
 
     try:
-        fob = open('/etc/debian_version')
-        verfile = fob.readline().strip()
-        fob.close()
+        with open('/etc/debian_version', errors='backslashreplace') as fob:
+            verfile = fob.readline().strip()
     except IOError:
         print('Unable to open /etc/debian_version', file=sys.stderr)
 
@@ -816,11 +810,12 @@ def generate_blank_report(package, pkgversion, severity, justification,
                           depinfo, confinfo, foundfile='', incfiles='',
                           system='debian', exinfo=None, type=None, klass='',
                           subject='', tags='', body='', mode=MODE_EXPERT,
-                          pseudos=None, debsumsoutput=None, issource=False):
+                          pseudos=None, debsumsoutput=None, issource=False,
+                          options=None):
     # For now...
     from . import bugreport
 
-    sysinfo = (package not in ('wnpp', 'ftp.debian.org'))
+    sysinfo = (package not in ('wnpp', 'ftp.debian.org') and (options and not options.buildd_format))
 
     # followup is where bugreport expects the notification of the bug reportbug
     # to follow-up, but reportbug pass this information with 'exinfo'
@@ -834,25 +829,6 @@ def generate_blank_report(package, pkgversion, severity, justification,
     return str(rep)
 
 
-def get_cpu_cores():
-    cpucount = 0
-    try:
-        fob = open('/proc/cpuinfo')
-    except IOError:
-        print('Unable to open /proc/cpuinfo', file=sys.stderr)
-        return 0
-
-    for line in fob:
-        if line.startswith('processor'):
-            cpucount += 1
-        # Alpha platform
-        if line.startswith('cpus detected'):
-            cpucount = int(line.split()[-1])
-    fob.close()
-
-    return max(cpucount, 1)
-
-
 class our_lex(shlex.shlex):
     def get_token(self):
         token = shlex.shlex.get_token(self)
@@ -872,7 +848,7 @@ CONFIG_ARGS = (
     'sign', 'nocc', 'nocompress', 'dontquery', 'noconf', 'mirrors', 'keyid',
     'headers', 'interface', 'template', 'mode', 'check_available', 'query_src',
     'printonly', 'offline', 'check_uid', 'smtptls', 'smtpuser', 'smtppasswd',
-    'paranoid', 'mbox_reader_cmd', 'max_attachment_size')
+    'paranoid', 'mbox_reader_cmd', 'max_attachment_size', 'listccme', 'outfile')
 
 
 class Mua:
@@ -963,7 +939,9 @@ def mua_exists(mua):
     output = '/dev/null'
     if os.path.exists(output):
         try:
-            returnvalue = subprocess.call(MUAVERSION[mua_tmp], stdout=open(output, 'w'), stderr=subprocess.STDOUT,
+            returnvalue = subprocess.call(MUAVERSION[mua_tmp],
+                                          stdout=open(output, 'w', errors='backslashreplace'),
+                                          stderr=subprocess.STDOUT,
                                           shell=True)
         except (IOError, OSError):
             returnvalue = subprocess.call(MUAVERSION[mua_tmp], shell=True)
@@ -1014,6 +992,8 @@ def parse_config_files():
                     args['nocc'] = (token == 'no-cc')
                 elif token in ('no-compress', 'compress'):
                     args['nocompress'] = (token == 'no-compress')
+                elif token in ('no-list-cc-me', 'list-cc-me'):
+                    args['listccme'] = (token == 'list-cc-me')
                 elif token in ('no-query-bts', 'query-bts'):
                     args['dontquery'] = (token == 'no-query-bts')
                 elif token in ('config-files', 'no-config-files'):
@@ -1023,7 +1003,7 @@ def parse_config_files():
                 elif token in ('email', 'realname', 'replyto', 'http_proxy',
                                'smtphost', 'editor', 'mua', 'mta', 'smtpuser',
                                'smtppasswd', 'justification', 'keyid',
-                               'mbox_reader_cmd'):
+                               'mbox_reader_cmd', 'outfile'):
                     bit = lex.get_token()
                     args[token] = bit
                 elif token in ('no-smtptls', 'smtptls'):
@@ -1038,6 +1018,8 @@ def parse_config_files():
                         args['sign'] = ''
                 elif token == 'ui':
                     token = lex.get_token().lower()
+                    if token == 'gtk2':
+                        token = 'gtk'
                     if token in list(AVAILABLE_UIS.keys()):
                         args['interface'] = token
                 elif token == 'mode':
@@ -1086,7 +1068,7 @@ def parse_bug_control_file(filename):
     submitas = submitto = None
     reportwith = []
     supplemental = []
-    fh = open(filename)
+    fh = open(filename, errors='backslashreplace')
     for line in fh:
         line = line.strip()
         parts = line.split(': ')
@@ -1156,7 +1138,7 @@ def cleanup_msg(dmessage, headers, pseudos, type):
                 continue
             else:
                 # Permit bogus headers in the pseudoheader section
-                headers.append(re.split(':\s+', line, 1))
+                headers.append(re.split(r':\s+', line, 1))
         elif line.strip() != NEWBIELINE:
             message += line + '\n'
 
@@ -1207,7 +1189,7 @@ def launch_mbox_reader(cmd, url, http_proxy, timeout):
     (fd, fname) = TempFile()
     try:
         for line in mbox.splitlines():
-            fd.write(line)
+            fd.write(line + '\n')
         fd.close()
         if cmd is not None:
             try:
@@ -1238,13 +1220,13 @@ def get_running_kernel_pkg():
         return None
 
 
-def exec_and_parse_bugscript(handler, bugscript):
+def exec_and_parse_bugscript(handler, bugscript, runner=os.system):
     """Execute and parse the output of the package bugscript, in particular
     identifying the headers and pseudo-headers blocks, if present"""
 
     fh, filename = TempFile()
     fh.close()
-    rc = os.system('LC_ALL=C %s %s %s' % (handler, pipes.quote(bugscript),
+    rc = runner('LC_ALL=C %s %s %s' % (handler, pipes.quote(bugscript),
                                           pipes.quote(filename)))
 
     isheaders = False
@@ -1288,7 +1270,7 @@ def check_package_name(pkg):
 
     Returns True if the package name is valid."""
 
-    pkg_re = re.compile('^[a-z0-9][a-z0-9+-\.]+$')
+    pkg_re = re.compile(r'^[a-z0-9][a-z0-9+-\.]+$')
 
     return True if pkg_re.match(pkg) else False
 
@@ -1302,11 +1284,81 @@ def get_init_system():
         init = 'systemd (via /run/systemd/system)'
     if not subprocess.call('. /lib/lsb/init-functions ; init_is_upstart', shell=True):
         init = 'upstart (via init_is_upstart())'
+    elif os.path.isfile('/run/runit.stopit'):
+        init = 'runit (via /run/runit.stopit)'
     elif os.path.isfile('/sbin/init') and not os.path.islink('/sbin/init'):
         init = 'sysvinit (via /sbin/init)'
 
     return init
 
+def get_lsm_info():
+    """Determines the linux security module enabled on the current machine
+
+    Returns None if there is no LSM enabled on the machine or if the state
+    cannot be determined."""
+
+    lsminfo = None
+
+    if os.path.exists('/usr/bin/aa-enabled') \
+       and (subprocess.call(['/usr/bin/aa-enabled', '--quiet']) == 0):
+        lsminfo = 'AppArmor: enabled'
+
+    if os.path.exists('/usr/sbin/selinuxenabled') and (subprocess.call(['/usr/sbin/selinuxenabled']) == 0):
+        if lsminfo is None:
+            lsminfo = 'SELinux: enabled - '
+        else:
+            lsminfo += '; SELinux: enabled - '
+        enforce_status = subprocess.check_output(['/usr/sbin/getenforce']).decode('ascii')
+        lsminfo += 'Mode: %s - ' % enforce_status[:-1]
+        with open('/etc/selinux/config', 'r') as f:
+            lines = f.readlines()
+            for line in lines:
+                if line.startswith('SELINUXTYPE='):
+                    lsminfo += 'Policy name: %s' % line.split('=')[1][:-1]
+                    break
+
+    return lsminfo
+
+
+def get_kernel_taint_flags():
+    """Determines the kernel taint flags"""
+
+    # https://github.com/torvalds/linux/blob/cedc5b6aab493f6b1b1d381dccc0cc082da7d3d8/include/linux/kernel.h#L582
+    # this is going to need updating (but maybe not that often)
+    TAINT_FLAGS = ['TAINT_PROPRIETARY_MODULE',
+                   'TAINT_FORCED_MODULE',
+                   'TAINT_CPU_OUT_OF_SPEC',
+                   'TAINT_FORCED_RMMOD',
+                   'TAINT_MACHINE_CHECK',
+                   'TAINT_BAD_PAGE',
+                   'TAINT_USER',
+                   'TAINT_DIE',
+                   'TAINT_OVERRIDDEN_ACPI_TABLE',
+                   'TAINT_WARN',
+                   'TAINT_CRAP',
+                   'TAINT_FIRMWARE_WORKAROUND',
+                   'TAINT_OOT_MODULE',
+                   'TAINT_UNSIGNED_MODULE',
+                   'TAINT_SOFTLOCKUP',
+                   'TAINT_LIVEPATCH',
+                   'TAINT_AUX',
+                   'TAINT_RANDSTRUCT',
+    ]
+
+    flags = []
+
+    if os.path.exists('/proc/sys/kernel/tainted'):
+        tainted = int(open('/proc/sys/kernel/tainted').read())
+
+        # tainted is an integer representing a bitmask, so logical-AND against the list of
+        # flags and if it's a TRUE, then append it to the list of flags enabled
+        for i, flag in enumerate(TAINT_FLAGS):
+            if tainted & 2**i:
+                flags.append(flag)
+
+    return flags
+
+
 def is_security_update(pkgname, pkgversion):
     """Determine whether a given package is a security update.
 
